/*****************************************************************************
/* trim_wtmp_func.c                                                          */
/*                                                                           */
/* Supplies machine independent functions for handling the utmp/wtmp files.  */
/*                                                                           */
/* (c) 1995 by Markus Schoepflin. See README for details.                    */
/* mod 1999 by Brian Huntley. See README for details                         */
/*****************************************************************************/

#include "trim_wtmp_func.h"

extern char *pname;

static S_UTMP utent;

/* The extended format stores the time in a 'struct timeval'. We are
   only interested in the number of seconds since 1970 which is what
   we have on other systems.
*/

#ifdef SUN5_X

#define UT_TIME ut_tv.tv_sec

#else

#define UT_TIME ut_time

#endif

/* SUN4 doesn't have difftime. (sigh) */

#ifdef SUN4

#define difftime(_time1,_time0) ((double)(_time1-_time0))

#endif

/* Not needed for SUN4 and ULTRIX because there is no type field in the
   utmp struct.
*/

#if defined(SUN5_X) || defined(SUN5) || defined(OSF) || defined(AIX)

static const char *psz_ut_type[] = {
  "EMPTY", "RUN_LVL", "BOOT_TIME", "OLD_TIME", "NEW_TIME", "INIT_PROCESS",
  "LOGIN_PROCESS", "USER_PROCESS", "DEAD_PROCESS", "ACCOUNTING" };

#elif LINUX

static const char *psz_ut_type[] = {
  "UT_UNKNOWN", "RUN_LVL", "BOOT_TIME", "NEW_TIME", "OLD_TIME",
  "INIT_PROCESS", "LOGIN_PROCESS", "USER_PROCESS", "DEAD_PROCESS"};

#endif

/* Prints a header for each field in the verbose output. Only those fields
   that are interesting enough (in my opinion) are printed.
*/

void my_printheader(FILE *outfile) {

#ifdef SUN5_X

  fprintf(outfile, "%32s %4s %32s %8s %13s %24s %32s\n\n",
	  "user", "id", "line", "pid", "type", "time", "host");

#elif SUN5

  fprintf(outfile, "%8s %4s %12s %5s %13s %24s\n\n",
	  "user", "id", "line", "pid", "type", "time");

#elif OSF

  fprintf(outfile, "%32s %14s %32s %13s %8s %24s %32s\n\n",
	  "user", "id", "line", "type", "pid", "time", "host");

#elif defined(SUN4) || defined(ULTRIX)

  fprintf(outfile, "%8s %8s %16s %24s\n\n",
	  "line", "user", "host", "time");

#elif LINUX

  fprintf(outfile, "%13s %8s %12s %2s %24s %8s %16s\n\n",
	  "type", "pid", "line", "id", "time", "user", "host");

#elif AIX

  fprintf(outfile, "%8s %14s %12s %8s %24s %32s\n",
	  "user", "line", "type", "pid", "time", "host");

#endif

}

void my_printutent(const S_UTMP *ent, FILE *outfile) {

#ifdef SUN5_X

  fprintf(outfile, "%32s %4.4s %32s %8lu %13s %24.24s %32.32s\n",
	  ent->ut_user, ent->ut_id, ent->ut_line,
	  ent->ut_pid, psz_ut_type[ent->ut_type],
	  ctime(&(ent->UT_TIME)), ent->ut_host);

#elif SUN5

  fprintf(outfile, "%8.8s %4.4s %12.12s %5hu %13.13s %24.24s\n",
	  ent->ut_user, ent->ut_id, ent->ut_line,
	  ent->ut_pid, psz_ut_type[ent->ut_type],
	  ctime(&(ent->UT_TIME)));

#elif OSF

  fprintf(outfile, "%32.32s %14.14s %32.32s %13.13s %8.8d %24.24s %32.32s\n",
	  ent->ut_user, ent->ut_id, ent->ut_line,
	  psz_ut_type[ent->ut_type], ent->ut_pid,
	  ctime(&(ent->UT_TIME)), ent->ut_host);
 
#elif defined(SUN4) || defined(ULTRIX)

  fprintf(outfile, "%8.8s %8.8s %16.16s %24.24s\n",
	  ent->ut_line, ent->ut_name, ent->ut_host,
	  ctime((time_t*)&(ent->UT_TIME)) );

#elif LINUX

  fprintf(outfile, "%13s %8lu %12.12s %2.2s %24.24s %8.8s %16.16s\n",
	  psz_ut_type[ent->ut_type], ent->ut_pid, ent->ut_line,
	  ent->ut_id, ctime(&(ent->UT_TIME)), ent->ut_user, ent->ut_host);
	  
#elif AIX

  fprintf(outfile, "%8.8s %14.14s %12.12s %8.8d %24.24s %32.32s\n",
	  ent->ut_user,  ent->ut_line,
	  psz_ut_type[ent->ut_type], ent->ut_pid,
	  ctime(&(ent->UT_TIME)), ent->ut_host);

#endif

}

/* Reads an entry from the file pointed to by the file handler 'utfile' and
   returns a pointer to it. Returns NULL in case of error or EOF. (Well,
   there isn't any real error checking but I hope it will work for now.)
*/

S_UTMP *my_getutent (FILE *utfile) {

  if ( 1 == fread(&utent, sizeof(utent), 1, utfile) ) {
    return &utent;
  } else {
    if (feof(utfile)) {
      return NULL;
    } else {
      perror(pname);
      exit(EIO);
    }
  }
}

/* Writes out an entry to the file pointed to by the file handler 'utfile'.
   Dies on error.
*/

void my_pututent (FILE *utfile) {

  if ( 1 != fwrite(&utent, sizeof(utent), 1, utfile) ) {
    perror(pname);
    exit(1);
  }

}
    
/* Returns TRUE if the timestamp of the supplied entry is the same or newer
   than the specified cutoff time.
*/

BOOL my_selectutent (const S_UTMP *utent, time_t cut_time, BOOL before) {
  
  if (before) {
    if (difftime(cut_time, utent->UT_TIME) >= 0.0) {
      return TRUE;
    } else {
      return FALSE;
    }
  } else {
    if (difftime(utent->UT_TIME, cut_time) >= 0.0) {
      return TRUE;
    } else {
      return FALSE;
    }
  }
}
